package com.milosev.neverendingapplication

import android.Manifest.permission.REQUEST_IGNORE_BATTERY_OPTIMIZATIONS
import android.annotation.SuppressLint
import android.app.Activity
import android.content.*
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.PowerManager
import android.provider.Settings
import android.view.View
import android.view.inputmethod.InputMethodManager
import android.widget.Button
import android.widget.TextView
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AppCompatActivity
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.milosev.justanotherrealtimetrackingapp.IntentAction
import com.milosev.justanotherrealtimetrackingapp.IntentExtras
import com.milosev.justanotherrealtimetrackingapp.SharedPreferencesStrings


class MainActivity : AppCompatActivity() {

    private var broadCastReceiver: BroadcastReceiver = MyBroadcastReceiver()
    private lateinit var sharedPreferences: SharedPreferences

    @RequiresApi(Build.VERSION_CODES.O)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        checkOptimization(this)
        sharedPreferences = getSharedPreferences(SharedPreferencesStrings.SERVICE_NAME, MODE_PRIVATE)

        val btnStop: Button = findViewById<View>(R.id.btnStop) as Button
        val btnStart: Button = findViewById<View>(R.id.btnStart) as Button
        val btnOpenBatteryOptimization: Button = findViewById<View>(R.id.btnOpenBatteryOptimization) as Button

        broadCastReceiver = registerBroadCastReceiver(btnStart, btnStop)

        enableStartButton(true, btnStart, btnStop)

        btnStart.setOnClickListener {
            enableStartButton(false, btnStart, btnStop)

            val inputMethodManager =
                getSystemService(Activity.INPUT_METHOD_SERVICE) as InputMethodManager
            inputMethodManager.hideSoftInputFromWindow(it.windowToken, 0)

            val numberOfTicks: TextView =
                findViewById<View>(R.id.textViewNumberOfTicks) as TextView
            numberOfTicks.text = "0"

            val component = ComponentName(this, MyBroadcastReceiver::class.java)
            packageManager.setComponentEnabledSetting(
                component,
                PackageManager.COMPONENT_ENABLED_STATE_ENABLED,
                PackageManager.DONT_KILL_APP
            )

            val numOfSecondsForTick: TextView =
                findViewById<View>(R.id.txtRequestUpdates) as TextView
            var strNumOfSecondsForTick: String = numOfSecondsForTick.text.toString()
            if (strNumOfSecondsForTick.isEmpty()) strNumOfSecondsForTick = "30"

            val intentStartMyService =
                Intent(this, MyService::class.java)
            intentStartMyService.action = IntentAction.START_MY_SERVICE
            intentStartMyService.putExtra(
                IntentExtras.NUM_OF_SECONDS_FOR_TICK,
                strNumOfSecondsForTick.toLong()
            )
            startForegroundService(intentStartMyService)

            val edit: SharedPreferences.Editor = sharedPreferences.edit()
            edit.putString(SharedPreferencesStrings.SERVICE_NAME, SharedPreferencesStrings.START)
            edit.commit()
        }

        btnStop.setOnClickListener {

            enableStartButton(true, btnStart, btnStop)

            val component = ComponentName(this, MyBroadcastReceiver::class.java)
            packageManager.setComponentEnabledSetting(
                component,
                PackageManager.COMPONENT_ENABLED_STATE_DISABLED,
                PackageManager.DONT_KILL_APP
            )

            val intentStopMyService = Intent(this, MyService::class.java)
            intentStopMyService.action = IntentAction.STOP_MY_SERVICE
            startForegroundService(intentStopMyService)
            val edit: SharedPreferences.Editor = sharedPreferences.edit()
            edit.putString(SharedPreferencesStrings.SERVICE_NAME, SharedPreferencesStrings.STOP)
            edit.commit()
        }

        btnOpenBatteryOptimization.setOnClickListener {
            openBatteryOptimization(this)
        }
    }

    private fun registerBroadCastReceiver(btnStart: Button, btnStop: Button): BroadcastReceiver {
        broadCastReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context, intent: Intent) {
                when (intent.action) {
                    IntentAction.NUM_OF_TICKS -> {
                        val numOfTicks = intent.getIntExtra (IntentExtras.NUM_OF_TICKS, 30)
                        val numberOfTicks: TextView =
                            findViewById<View>(R.id.textViewNumberOfTicks) as TextView
                        numberOfTicks.text = numOfTicks.toString()

                        enableStartButton(false, btnStart, btnStop)
                    }
                }
            }
        }

        LocalBroadcastManager.getInstance(this)
            .registerReceiver(broadCastReceiver, IntentFilter(IntentAction.NUM_OF_TICKS))

        return broadCastReceiver
    }

    @RequiresApi(Build.VERSION_CODES.O)
    override fun onResume() {
        super.onResume()
        val isServiceStarted = sharedPreferences.getString(SharedPreferencesStrings.SERVICE_NAME, SharedPreferencesStrings.STOP)
        if (isServiceStarted == SharedPreferencesStrings.START) {
            LocalBroadcastManager.getInstance(this)
                .registerReceiver(broadCastReceiver, IntentFilter(IntentAction.NUM_OF_TICKS))

            val intent = Intent(this, MyService::class.java)
            intent.action = IntentAction.START_MY_SERVICE
            startForegroundService(intent)
        }
    }

    fun enableStartButton(enable: Boolean, btnStart: Button, btnStop: Button) {
        btnStart.isEnabled = enable
        btnStop.isEnabled = !btnStart.isEnabled
    }

    @SuppressLint("NewApi", "BatteryLife")
    private fun checkOptimization(context: Context) {
        val packageName = applicationContext.packageName
        val pm = applicationContext.getSystemService(POWER_SERVICE) as PowerManager
        if (!pm.isIgnoringBatteryOptimizations(packageName)) {
            val intent = Intent()
            intent.action = Settings.ACTION_REQUEST_IGNORE_BATTERY_OPTIMIZATIONS
            intent.data = Uri.parse("package:" + context.packageName)
            context.startActivity(intent)
        }
    }

    private fun openBatteryOptimization(context: Context) =
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            val intent = Intent()
            intent.action = Settings.ACTION_IGNORE_BATTERY_OPTIMIZATION_SETTINGS
            context.startActivity(intent)
        } else {
            //Timber.d("Battery optimization not necessary")
        }
}