package com.milosev.neverendingapplication

import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.Service
import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.os.Binder
import android.os.Build
import android.os.IBinder
import android.os.PowerManager
import androidx.annotation.RequiresApi
import androidx.core.app.NotificationCompat
import com.milosev.justanotherrealtimetrackingapp.IntentAction
import com.milosev.justanotherrealtimetrackingapp.IntentExtras
import com.milosev.justanotherrealtimetrackingapp.WakeLock
import kotlinx.coroutines.*

class MyService : Service(),CoroutineScope by MainScope() {

    private var job: Job? = null

    override fun onBind(intent: Intent): IBinder {
        return Binder()
    }

    @RequiresApi(Build.VERSION_CODES.O)
    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        when (intent?.action) {
            IntentAction.START_MY_SERVICE -> {

                val channelId = createNotificationChannel("my_service", "My Background Service")
                val notificationBuilder = NotificationCompat.Builder(this, channelId)
                val notification = notificationBuilder.setOngoing(true)
                    .setContentTitle("test")
                    .setContentText("test")
                    .setSmallIcon(R.mipmap.ic_launcher)
                    .setPriority(1)
                    .setCategory(Notification.CATEGORY_SERVICE)
                    .build()

                startForeground(101, notification)
                val context = this
                val numOfSecondsForTick = intent.getLongExtra (IntentExtras.NUM_OF_SECONDS_FOR_TICK, 30)
                var numOfTicks = 0
                job = launch {
                    while(true) {
                        val myIntent = Intent(context, MyBroadcastReceiver::class.java).setAction(IntentAction.NUM_OF_TICKS)
                        myIntent.putExtra("numOfTicks", numOfTicks)
                        numOfTicks += 1

                        sendBroadcast(myIntent)
                        delay(numOfSecondsForTick * 1_000)
                    }
                }
            }
            IntentAction.STOP_MY_SERVICE -> {
                job?.cancel()
                releaseWakelock()
                stopForeground(true)
                stopSelfResult(startId)
            }
        }
        return START_STICKY
    }

    @RequiresApi(Build.VERSION_CODES.O)
    private fun createNotificationChannel(channelId: String, channelName: String): String {
        val chan = NotificationChannel(
            channelId,
            channelName, NotificationManager.IMPORTANCE_LOW
        )
        chan.lightColor = Color.RED
        chan.lockscreenVisibility = Notification.VISIBILITY_PRIVATE
        val service = getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        service.createNotificationChannel(chan)
        return channelId
    }

    override fun onCreate() {
        super.onCreate()
        acquireWakelock()
    }

    override fun onDestroy() {
        super.onDestroy()
        val restartIntent = Intent(this, MyBroadcastReceiver::class.java).setAction(IntentAction.RESTART_MY_SERVICE)
        sendBroadcast(restartIntent)
    }

    override fun onTaskRemoved(rootIntent: Intent?) {
        super.onTaskRemoved(rootIntent)
        val restartIntent = Intent(this, MyBroadcastReceiver::class.java).setAction(IntentAction.RESTART_MY_SERVICE)
        sendBroadcast(restartIntent)
    }

    private val wakeLock: PowerManager.WakeLock by lazy {
        (getSystemService(Context.POWER_SERVICE) as PowerManager).run {
            newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, WakeLock.TAG)
        }
    }

    private fun acquireWakelock() {
        try {
            wakeLock.let {
                wakeLock.setReferenceCounted(false)
                if (!wakeLock.isHeld) {
                    wakeLock.acquire()
                }
            }
        } catch (e: RuntimeException) {
        }
    }

    private fun releaseWakelock() {
        try {
            wakeLock.let {
                if (it.isHeld) {
                    it.release()
                }
            }
        } catch (e: RuntimeException) {
        }
    }

}